@@ -28,8 +28,10 @@
 #include <limits.h>
 #include <stdarg.h>
 #include <mntent.h>
+#include <inttypes.h>
+#include <pthread.h>
 
-#include "apparmor.h"
+#include <sys/apparmor.h>
 
 /* some non-Linux systems do not define a static value */
 #ifndef PATH_MAX
@@ -149,13 +151,40 @@ static char *procattr_path(pid_t pid, const char *attr)
 	return NULL;
 }
 
+/**
+ * parse_confinement_mode - get the mode from the confinement string
+ * @con: the confinement string
+ * @size: size of the confinement string
+ *
+ * Modifies con to NUL-terminate the label string and the mode string.
+ *
+ * Returns: a pointer to the NUL-terminated mode inside the confinement string
+ * or NULL if the mode was not found
+ */
+static char *parse_confinement_mode(char *con, int size)
+{
+	if (strcmp(con, "unconfined") != 0 &&
+	    size > 4 && con[size - 2] == ')') {
+		int pos = size - 3;
+
+		while (pos > 0 && !(con[pos] == ' ' && con[pos + 1] == '('))
+			pos--;
+		if (pos > 0) {
+			con[pos] = 0; /* overwrite ' ' */
+			con[size - 2] = 0; /* overwrite trailing ) */
+			return &con[pos + 2]; /* skip '(' */
+		}
+	}
+	return NULL;
+}
+
 /**
  * aa_getprocattr_raw - get the contents of @attr for @tid into @buf
  * @tid: tid of task to query
  * @attr: which /proc/<tid>/attr/<attr> to query
  * @buf: buffer to store the result in
  * @len: size of the buffer
- * @mode: if set will point to mode string within buffer if it is present
+ * @mode: if non-NULL and a mode is present, will point to mode string in @buf
  *
  * Returns: size of data read or -1 on error, and sets errno
  */
@@ -165,6 +194,7 @@ int aa_getprocattr_raw(pid_t tid, const char *attr, char *buf, int len,
 	int rc = -1;
 	int fd, ret;
 	char *tmp = NULL;
+	char *mode_str;
 	int size = 0;
 
 	if (!buf || len <= 0) {
@@ -217,25 +247,9 @@ int aa_getprocattr_raw(pid_t tid, const char *attr, char *buf, int len,
 			size++;
 		}
 
-		/*
-		 * now separate the mode.  If we don't find it just
-		 * return NULL
-		 */
+		mode_str = parse_confinement_mode(buf, size);
 		if (mode)
-			*mode = NULL;
-		if (strcmp(buf, "unconfined") != 0 &&
-		    size > 4 && buf[size - 2] == ')') {
-			int pos = size - 3;
-			while (pos > 0 &&
-			       !(buf[pos] == ' ' && buf[pos + 1] == '('))
-				pos--;
-			if (pos > 0) {
-				buf[pos] = 0; /* overwrite ' ' */
-				buf[size - 2] = 0; /* overwrite trailing ) */
-				if (mode)
-					*mode = &buf[pos + 2]; /* skip '(' */
-			}
-		}
+			*mode = mode_str;
 	}
 	rc = size;
 
@@ -251,17 +265,24 @@ out:
  * aa_getprocattr - get the contents of @attr for @tid into @buf
  * @tid: tid of task to query
  * @attr: which /proc/<tid>/attr/<attr> to query
- * @buf: allocated buffer the result is stored in
- * @mode: if set will point to mode string within buffer if it is present
+ * @con: allocated buffer the result is stored in
+ * @mode: if non-NULL and a mode is present, will point to mode string in @con
  *
  * Returns: size of data read or -1 on error, and sets errno
+ *
+ * Guarantees that @con and @mode are null terminated.  The length returned
+ * is for all data including both @con and @mode, and maybe > than strlen(@con)
+ * even if @mode is NULL
+ *
+ * Caller is responsible for freeing the buffer returned in @con.  @mode is
+ * always contained within @con's buffer and so NEVER do free(@mode)
  */
-int aa_getprocattr(pid_t tid, const char *attr, char **buf, char **mode)
+int aa_getprocattr(pid_t tid, const char *attr, char **con, char **mode)
 {
 	int rc, size = INITIAL_GUESS_SIZE/2;
 	char *buffer = NULL;
 
-	if (!buf) {
+	if (!con) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -278,10 +299,11 @@ int aa_getprocattr(pid_t tid, const char *attr, char **buf, char **mode)
 
 	if (rc == -1) {
 		free(buffer);
-		*buf = NULL;
-		*mode = NULL;
+		*con = NULL;
+		if (mode)
+			*mode = NULL;
 	} else
-		*buf = buffer;
+		*con = buffer;
 
 	return rc;
 }
@@ -333,7 +355,7 @@ int aa_change_hat(const char *subprofile, unsigned long token)
 	int rc = -1;
 	int len = 0;
 	char *buf = NULL;
-	const char *fmt = "changehat %016x^%s";
+	const char *fmt = "changehat %016lx^%s";
 
 	/* both may not be null */
 	if (!(token || subprofile)) {
@@ -508,11 +530,11 @@ int (aa_change_hat_vargs)(unsigned long token, int nhats, ...)
  * aa_gettaskcon - get the confinement for task @target in an allocated buffer
  * @target: task to query
  * @con: pointer to returned buffer with the confinement string
- * @mode: if provided will point to the mode string in @con if present
+ * @mode: if non-NULL and a mode is present, will point to mode string in @con
  *
  * Returns: length of confinement data or -1 on error and sets errno
  *
- * Guarentees that @con and @mode are null terminated.  The length returned
+ * Guarantees that @con and @mode are null terminated.  The length returned
  * is for all data including both @con and @mode, and maybe > than strlen(@con)
  * even if @mode is NULL
  *
@@ -527,11 +549,11 @@ int aa_gettaskcon(pid_t target, char **con, char **mode)
 /**
  * aa_getcon - get the confinement for current task in an allocated buffer
  * @con: pointer to return buffer with the confinement if successful
- * @mode: if provided will point to the mode string in @con if present
+ * @mode: if non-NULL and a mode is present, will point to mode string in @con
  *
  * Returns: length of confinement data or -1 on error and sets errno
  *
- * Guarentees that @con and @mode are null terminated.  The length returned
+ * Guarantees that @con and @mode are null terminated.  The length returned
  * is for all data including both @con and @mode, and may > than strlen(@con)
  * even if @mode is NULL
  *
@@ -551,40 +573,49 @@ int aa_getcon(char **con, char **mode)
 /**
  * aa_getpeercon_raw - get the confinement of the socket's peer (other end)
  * @fd: socket to get peer confinement for
- * @con: pointer to buffer to store confinement string
- * @size: initially contains size of the buffer, returns size of data read
+ * @buf: buffer to store the result in
+ * @len: initially contains size of the buffer, returns size of data read
+ * @mode: if non-NULL and a mode is present, will point to mode string in @buf
  *
  * Returns: length of confinement data including null termination or -1 on error
- *          if errno == ERANGE then @size will hold the size needed
+ *          if errno == ERANGE then @len will hold the size needed
  */
-int aa_getpeercon_raw(int fd, char *buffer, int *size)
+int aa_getpeercon_raw(int fd, char *buf, int *len, char **mode)
 {
-	socklen_t optlen = *size;
+	socklen_t optlen = *len;
+	char *mode_str;
 	int rc;
 
-	if (optlen <= 0 || buffer == NULL) {
+	if (optlen <= 0 || buf == NULL) {
 		errno = EINVAL;
 		return -1;
 	}
 
-	rc = getsockopt(fd, SOL_SOCKET, SO_PEERSEC, buffer, &optlen);
+	rc = getsockopt(fd, SOL_SOCKET, SO_PEERSEC, buf, &optlen);
 	if (rc == -1 || optlen <= 0)
 		goto out;
 
 	/* check for null termination */
-	if (buffer[optlen - 1] != 0) {
-		if (optlen < *size) {
-			buffer[optlen] = 0;
+	if (buf[optlen - 1] != 0) {
+		if (optlen < *len) {
+			buf[optlen] = 0;
 			optlen++;
 		} else {
-			/* buffer needs to be bigger by 1 */
+			/* buf needs to be bigger by 1 */
 			rc = -1;
 			errno = ERANGE;
 			optlen++;
+			goto out;
 		}
 	}
+
+	mode_str = parse_confinement_mode(buf, optlen);
+	if (mode)
+		*mode = mode_str;
+
+	rc = optlen;
 out:
-	*size = optlen;
+	*len = optlen;
 	return rc;
 }
 
@@ -592,14 +623,20 @@ out:
  * aa_getpeercon - get the confinement of the socket's peer (other end)
  * @fd: socket to get peer confinement for
  * @con: pointer to allocated buffer with the confinement string
+ * @mode: if non-NULL and a mode is present, will point to mode string in @con
  *
  * Returns: length of confinement data including null termination or -1 on error
  *
- * Caller is responsible for freeing the buffer returned.
+ * Guarantees that @con and @mode are null terminated.  The length returned
+ * is for all data including both @con and @mode, and maybe > than strlen(@con)
+ * even if @mode is NULL
+ *
+ * Caller is responsible for freeing the buffer returned in @con.  @mode is
+ * always contained within @con's buffer and so NEVER do free(@mode)
  */
-int aa_getpeercon(int fd, char **con)
+int aa_getpeercon(int fd, char **con, char **mode)
 {
-	int rc, size = INITIAL_GUESS_SIZE;
+	int rc, last_size, size = INITIAL_GUESS_SIZE;
 	char *buffer = NULL;
 
 	if (!con) {
@@ -608,20 +645,134 @@ int aa_getpeercon(int fd, char **con)
 	}
 
 	do {
+		last_size = size;
 		buffer = realloc(buffer, size);
 		if (!buffer)
 			return -1;
 		memset(buffer, 0, size);
 
-		rc = aa_getpeercon_raw(fd, buffer, &size);
-	} while (rc == -1 && errno == ERANGE);
+		rc = aa_getpeercon_raw(fd, buffer, &size, mode);
+		/* size should contain actual size needed if errno == ERANGE */
+	} while (rc == -1 && errno == ERANGE && size > last_size);
 
 	if (rc == -1) {
 		free(buffer);
 		*con = NULL;
+		if (mode)
+			*mode = NULL;
 		size = -1;
 	} else
 		*con = buffer;
 
 	return size;
 }
+
+static pthread_once_t aafs_access_control = PTHREAD_ONCE_INIT;
+static char *aafs_access = NULL;
+
+static void aafs_access_init_once(void)
+{
+	char *aafs;
+	int ret;
+
+	ret = aa_find_mountpoint(&aafs);
+	if (ret < 0)
+		return;
+
+	ret = asprintf(&aafs_access, "%s/.access", aafs);
+	if (ret < 0)
+		aafs_access = NULL;
+
+	free(aafs);
+}
+
+/* "allow 0x00000000\ndeny 0x00000000\naudit 0x00000000\nquiet 0x00000000\n" */
+#define QUERY_LABEL_REPLY_LEN	67
+
+/**
+ * aa_query_label - query the access(es) of a label
+ * @mask: permission bits to query
+ * @query: binary query string, must be offset by AA_QUERY_CMD_LABEL_SIZE
+ * @size: size of the query string must include AA_QUERY_CMD_LABEL_SIZE
+ * @allowed: upon successful return, will be 1 if query is allowed and 0 if not
+ * @audited: upon successful return, will be 1 if query should be audited and 0
+ *           if not
+ *
+ * Returns: 0 on success else -1 and sets errno. If -1 is returned and errno is
+ *          ENOENT, the subject label in the query string is unknown to the
+ *          kernel.
+ */
+int query_label(uint32_t mask, char *query, size_t size, int *allowed,
+		int *audited)
+{
+	char buf[QUERY_LABEL_REPLY_LEN];
+	uint32_t allow, deny, audit, quiet;
+	int fd, ret, saved;
+
+	if (!mask || size <= AA_QUERY_CMD_LABEL_SIZE) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	ret = pthread_once(&aafs_access_control, aafs_access_init_once);
+	if (ret) {
+		errno = EINVAL;
+		return -1;
+	} else if (!aafs_access) {
+		errno = ENOMEM;
+		return -1;
+	}
+
+	fd = open(aafs_access, O_RDWR);
+	if (fd == -1) {
+		if (errno == ENOENT)
+			errno = EPROTONOSUPPORT;
+		return -1;
+	}
+
+	memcpy(query, AA_QUERY_CMD_LABEL, AA_QUERY_CMD_LABEL_SIZE);
+	errno = 0;
+	ret = write(fd, query, size);
+	if (ret != size) {
+		if (ret >= 0)
+			errno = EPROTO;
+		/* IMPORTANT: This is the only valid error path that can have
+		 * errno set to ENOENT. It indicates that the subject label
+		 * could not be found by the kernel.
+		 */
+		(void)close(fd);
+		return -1;
+	}
+
+	ret = read(fd, buf, QUERY_LABEL_REPLY_LEN);
+	saved = errno;
+	(void)close(fd);
+	errno = saved;
+	if (ret != QUERY_LABEL_REPLY_LEN) {
+		errno = EPROTO;
+		return -1;
+	}
+
+	ret = sscanf(buf, "allow 0x%8" SCNx32 "\n"
+			  "deny 0x%8"  SCNx32 "\n"
+			  "audit 0x%8" SCNx32 "\n"
+			  "quiet 0x%8" SCNx32 "\n",
+		     &allow, &deny, &audit, &quiet);
+	if (ret != 4) {
+		errno = EPROTONOSUPPORT;
+		return -1;
+	}
+
+	*allowed = mask & ~(allow & ~deny) ? 0 : 1;
+	if (!(*allowed))
+		audit = 0xFFFFFFFF;
+	*audited = mask & ~(audit & ~quiet) ? 0 : 1;
+
+	return 0;
+}
+
+/* export multiple aa_query_label symbols to compensate for downstream
+ * releases with differing symbol versions. */
+extern typeof((query_label)) __aa_query_label __attribute__((alias ("query_label")));
+symbol_version(__aa_query_label, aa_query_label, APPARMOR_1.1);
+default_symbol_version(query_label, aa_query_label, APPARMOR_2.9);
