@@ -25,6 +25,7 @@
 
 #include "immunix.h"
 #include "parser.h"
+#include "profile.h"
 
 struct alias_rule {
 	char *from;
@@ -50,7 +51,7 @@ int new_alias(const char *from, const char *to)
 {
 	struct alias_rule *alias, **result;
 
-	alias = calloc(1, sizeof(struct alias_rule));
+	alias = (struct alias_rule *) calloc(1, sizeof(struct alias_rule));
 	if (!alias) {
 		PERROR("Failed to allocate memory: %s\n", strerror(errno));
 		goto fail;
@@ -95,17 +96,17 @@ fail:
 static char *do_alias(struct alias_rule *alias, const char *target)
 {
 	int len = strlen(target) - strlen(alias->from) + strlen(alias->to);
-	char *new = malloc(len + 1);
-	if (!new) {
+	char *n = (char *) malloc(len + 1);
+	if (!n) {
 		PERROR("Failed to allocate memory: %s\n", strerror(errno));
 		return NULL;
 	}
-	sprintf(new, "%s%s", alias->to, target + strlen(alias->from));
+	sprintf(n, "%s%s", alias->to, target + strlen(alias->from));
 /*fprintf(stderr, "replaced alias: from: %s, to: %s, name: %s\n  %s\n", alias->from, alias->to, target, new);*/
-	return new;
+	return n;
 }
 
-static struct codomain *target_cod;
+static Profile *target_prof;
 static struct cod_entry *target_list;
 static void process_entries(const void *nodep, VISIT value, int __unused level)
 {
@@ -119,26 +120,25 @@ static void process_entries(const void *nodep, VISIT value, int __unused level)
 	len = strlen((*t)->from);
 
 	list_for_each(target_list, entry) {
-		if (entry->mode & (AA_SHARED_PERMS & AA_PTRACE_PERMS) ||
-		    entry->alias_ignore)
+		if ((entry->mode & AA_SHARED_PERMS) || entry->alias_ignore)
 			continue;
 		if (entry->name && strncmp((*t)->from, entry->name, len) == 0) {
-			char *new = do_alias(*t, entry->name);
-			if (!new)
+			char *n = do_alias(*t, entry->name);
+			if (!n)
 				return;
 			dup = copy_cod_entry(entry);
 			free(dup->name);
-			dup->name = new;
+			dup->name = n;
 		}
 		if (entry->link_name &&
 		    strncmp((*t)->from, entry->link_name, len) == 0) {
-			char *new = do_alias(*t, entry->link_name);
-			if (!new)
+			char *n = do_alias(*t, entry->link_name);
+			if (!n)
 				return;
 			if (!dup)
 				dup = copy_cod_entry(entry);
 			free(dup->link_name);
-			dup->link_name = new;
+			dup->link_name = n;
 		}
 		if (dup) {
 			dup->alias_ignore = 1;
@@ -152,11 +152,10 @@ static void process_entries(const void *nodep, VISIT value, int __unused level)
 	}
 }
 
-static struct codomain *target_cod;
 static void process_name(const void *nodep, VISIT value, int __unused level)
 {
 	struct alias_rule **t = (struct alias_rule **) nodep;
-	struct codomain *cod = target_cod;
+	Profile *prof = target_prof;
 	char *name;
 	int len;
 
@@ -165,36 +164,40 @@ static void process_name(const void *nodep, VISIT value, int __unused level)
 
 	len = strlen((*t)->from);
 
-	if (cod->attachment)
-		name = cod->attachment;
+	if (prof->attachment)
+		name = prof->attachment;
 	else
-		name = cod->name;
+		name = prof->name;
 
 	if (name && strncmp((*t)->from, name, len) == 0) {
 		struct alt_name *alt;
-		char *new = do_alias(*t, name);
-		if (!new)
+		char *n = do_alias(*t, name);
+		if (!n)
 			return;
 		/* aliases create alternate names */
-		alt = calloc(1, sizeof(struct alt_name));
-		if (!alt)
+		alt = (struct alt_name *) calloc(1, sizeof(struct alt_name));
+		if (!alt) {
+			free(n);
 			return;
-		alt->name = new;
-		alt->next = cod->altnames;
-		cod->altnames = alt;
+		}
+		alt->name = n;
+		alt->next = prof->altnames;
+		prof->altnames = alt;
 	}
 }
 
-void replace_aliases(struct codomain *cod)
+int replace_profile_aliases(Profile *prof)
 {
-	target_cod = cod;
+	target_prof = prof;
 	twalk(alias_table, process_name);
 
-	if (cod->entries) {
-		target_list = cod->entries;
-		target_cod = cod;
+	if (prof->entries) {
+		target_list = prof->entries;
+		target_prof = prof;
 		twalk(alias_table, process_entries);
 	}
+
+	return 0;
 }
 
 static void free_alias(void *nodep)
