@@ -2,6 +2,9 @@
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *   NOVELL (All rights reserved)
  *
+ *   Copyright (c) 2013
+ *   Canonical Ltd. (All rights reserved)
+ *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of version 2 of the GNU General Public
  *   License published by the Free Software Foundation.
@@ -25,6 +28,7 @@
 #define _(s) gettext(s)
 
 #include "parser.h"
+#include "profile.h"
 #include "libapparmor_re/apparmor_re.h"
 
 #include <unistd.h>
@@ -57,11 +61,9 @@
 #define SD_CODE_SIZE (sizeof(u8))
 #define SD_STR_LEN (sizeof(u16))
 
-#define SUBDOMAIN_INTERFACE_VERSION 2
 #define SUBDOMAIN_INTERFACE_DFA_VERSION 5
-#define SUBDOMAIN_INTERFACE_POLICY_DB 16
 
-int sd_serialize_codomain(int option, struct codomain *cod);
+int __sd_serialize_profile(int option, Profile *prof);
 
 static void print_error(int error)
 {
@@ -102,30 +104,30 @@ static void print_error(int error)
 	}
 }
 
-int load_codomain(int option, struct codomain *cod)
+int load_profile(int option, Profile *prof)
 {
 	int retval = 0;
 	int error = 0;
 
-	PDEBUG("Serializing policy for %s.\n", cod->name);
-	retval = sd_serialize_codomain(option, cod);
+	PDEBUG("Serializing policy for %s.\n", prof->name);
+	retval = __sd_serialize_profile(option, prof);
 
 	if (retval < 0) {
 		error = retval;	/* yeah, we'll just report the last error */
 		switch (option) {
 		case OPTION_ADD:
 			PERROR(_("%s: Unable to add \"%s\".  "),
-			       progname, cod->name);
+			       progname, prof->name);
 			print_error(error);
 			break;
 		case OPTION_REPLACE:
 			PERROR(_("%s: Unable to replace \"%s\".  "),
-			       progname, cod->name);
+			       progname, prof->name);
 			print_error(error);
 			break;
 		case OPTION_REMOVE:
 			PERROR(_("%s: Unable to remove \"%s\".  "),
-			       progname, cod->name);
+			       progname, prof->name);
 			print_error(error);
 			break;
 		case OPTION_STDOUT:
@@ -146,15 +148,15 @@ int load_codomain(int option, struct codomain *cod)
 		switch (option) {
 		case OPTION_ADD:
 			printf(_("Addition succeeded for \"%s\".\n"),
-			       cod->name);
+			       prof->name);
 			break;
 		case OPTION_REPLACE:
 			printf(_("Replacement succeeded for \"%s\".\n"),
-			       cod->name);
+			       prof->name);
 			break;
 		case OPTION_REMOVE:
 			printf(_("Removal succeeded for \"%s\".\n"),
-			       cod->name);
+			       prof->name);
 			break;
 		case OPTION_STDOUT:
 		case OPTION_OFILE:
@@ -209,17 +211,17 @@ const char *sd_code_names[] = {
 /* Currently we will just use a contiguous block of memory
    be we are going to just hide this for the moment.  */
 struct __sdserialize {
-	void *buffer;
-	void *pos;
-	void *extent;
+	u8 *buffer;
+	u8 *pos;
+	u8 *extent;
 };
 
 sd_serialize *alloc_sd_serial(void)
 {
-	sd_serialize *p = calloc(1, sizeof(sd_serialize));
+	sd_serialize *p = (sd_serialize *) calloc(1, sizeof(sd_serialize));
 	if (!p)
 		return NULL;
-	p->buffer = malloc(BUFFERINC);
+	p->buffer = (u8 *) malloc(BUFFERINC);
 	if (!p->buffer) {
 		free(p);
 		return NULL;
@@ -257,7 +259,7 @@ static inline void sd_inc(sd_serialize *p, int size)
 
 inline long sd_serial_size(sd_serialize *p)
 {
-	return (p->pos - p->buffer);
+	return (long) (p->pos) - (long) (p->buffer);
 }
 
 /* routines for writing data to the serialization buffer */
@@ -267,14 +269,14 @@ inline int sd_prepare_write(sd_serialize *p, enum sd_code code, size_t size)
 	if (p->pos + SD_CODE_SIZE + size > p->extent) {
 		long pos;
 		/* try and reallocate the buffer */
-		void *buffer = malloc(p->extent - p->buffer + (BUFFERINC * num));
-		memcpy(buffer, p->buffer, p->extent - p->buffer);
+		u8 *buffer = (u8 *) malloc((long)(p->extent) - (long)(p->buffer) + (BUFFERINC * num));
+		memcpy(buffer, p->buffer, (long)(p->extent) - (long)(p->buffer));
 
-		pos = p->pos - p->buffer;
+		pos = (long)(p->pos) - (long)(p->buffer);
 		if (buffer == NULL || errno == ENOMEM)
 			return 0;
 
-		p->extent = buffer + (p->extent - p->buffer) + (BUFFERINC * num);
+		p->extent = buffer + ((long)(p->extent) - (long)(p->buffer)) + (BUFFERINC * num);
 		free(p->buffer);
 		p->buffer = buffer;
 		p->pos = buffer + pos;
@@ -328,7 +330,7 @@ inline int sd_write64(sd_serialize *p, u64 b)
 	return 1;
 }
 
-inline int sd_write_name(sd_serialize *p, char *name)
+inline int sd_write_name(sd_serialize *p, const char *name)
 {
 	long size = 0;
 	PDEBUG("Writing name '%s'\n", name);
@@ -361,15 +363,15 @@ inline int sd_write_blob(sd_serialize *p, void *b, int buf_size, char *name)
 	return 1;
 }
 
-#define align64(X) (((size_t) (X) + (size_t) 7) & ~((size_t) 7))
+#define align64(X) (((X) + (typeof(X)) 7) & ~((typeof(X)) 7))
 inline int sd_write_aligned_blob(sd_serialize *p, void *b, int buf_size,
-				 char *name)
+				 const char *name)
 {
 	size_t pad;
 	u32 tmp;
 	if (!sd_write_name(p, name))
 		return 0;
-	pad = align64((p->pos + 5) - p->buffer) - ((p->pos + 5) - p->buffer);
+	pad = align64(p->pos + 5 - p->buffer) - (p->pos + 5 - p->buffer);
 	if (!sd_prepare_write(p, SD_BLOB, 4 + buf_size + pad))
 		return 0;
 	tmp = cpu_to_le32(buf_size + pad);
@@ -382,7 +384,7 @@ inline int sd_write_aligned_blob(sd_serialize *p, void *b, int buf_size,
 	return 1;
 }
 
-static int sd_write_strn(sd_serialize *p, char *b, int size, char *name)
+static int sd_write_strn(sd_serialize *p, char *b, int size, const char *name)
 {
 	u16 tmp;
 	if (!sd_write_name(p, name))
@@ -397,12 +399,12 @@ static int sd_write_strn(sd_serialize *p, char *b, int size, char *name)
 	return 1;
 }
 
-inline int sd_write_string(sd_serialize *p, char *b, char *name)
+inline int sd_write_string(sd_serialize *p, char *b, const char *name)
 {
 	return sd_write_strn(p, b, strlen(b) + 1, name);
 }
 
-inline int sd_write_struct(sd_serialize *p, char *name)
+inline int sd_write_struct(sd_serialize *p, const char *name)
 {
 	if (!sd_write_name(p, name))
 		return 0;
@@ -418,7 +420,7 @@ inline int sd_write_structend(sd_serialize *p)
 	return 1;
 }
 
-inline int sd_write_array(sd_serialize *p, char *name, int size)
+inline int sd_write_array(sd_serialize *p, const char *name, int size)
 {
 	u16 tmp;
 	if (!sd_write_name(p, name))
@@ -438,7 +440,7 @@ inline int sd_write_arrayend(sd_serialize *p)
 	return 1;
 }
 
-inline int sd_write_list(sd_serialize *p, char *name)
+inline int sd_write_list(sd_serialize *p, const char *name)
 {
 	if (!sd_write_name(p, name))
 		return 0;
@@ -546,7 +548,7 @@ int count_tailglob_ents(struct cod_entry *list)
 	return count;
 }
 
-int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
+int sd_serialize_profile(sd_serialize *p, Profile *profile,
 			 int flattened)
 {
 	uint64_t allowed_caps;
@@ -557,7 +559,7 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 		assert(profile->parent);
 		int res;
 
-		char *name = malloc(3 + strlen(profile->name) +
+		char *name = (char *) malloc(3 + strlen(profile->name) +
 				    strlen(profile->parent->name));
 		if (!name)
 			return 0;
@@ -573,7 +575,7 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 
 	/* only emit this if current kernel at least supports "create" */
 	if (perms_create) {
-		if (regex_type == AARE_DFA && profile->xmatch) {
+		if (profile->xmatch) {
 			if (!sd_serialize_dfa(p, profile->xmatch, profile->xmatch_size))
 				return 0;
 			if (!sd_write32(p, profile->xmatch_len))
@@ -610,12 +612,12 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 
 #define low_caps(X) ((u32) ((X) & 0xffffffff))
 #define high_caps(X) ((u32) (((X) >> 32) & 0xffffffff))
-	allowed_caps = (profile->capabilities) & ~profile->deny_caps;
+	allowed_caps = (profile->caps.allow) & ~profile->caps.deny;
 	if (!sd_write32(p, low_caps(allowed_caps)))
 		return 0;
-	if (!sd_write32(p, low_caps(allowed_caps & profile->audit_caps)))
+	if (!sd_write32(p, low_caps(allowed_caps & profile->caps.audit)))
 		return 0;
-	if (!sd_write32(p, low_caps(profile->deny_caps & profile->quiet_caps)))
+	if (!sd_write32(p, low_caps(profile->caps.deny & profile->caps.quiet)))
 		return 0;
 	if (!sd_write32(p, 0))
 		return 0;
@@ -624,9 +626,9 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 		return 0;
 	if (!sd_write32(p, high_caps(allowed_caps)))
 		return 0;
-	if (!sd_write32(p, high_caps(allowed_caps & profile->audit_caps)))
+	if (!sd_write32(p, high_caps(allowed_caps & profile->caps.audit)))
 		return 0;
-	if (!sd_write32(p, high_caps(profile->deny_caps & profile->quiet_caps)))
+	if (!sd_write32(p, high_caps(profile->caps.deny & profile->caps.quiet)))
 		return 0;
 	if (!sd_write32(p, 0))
 		return 0;
@@ -636,74 +638,52 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 	if (!sd_serialize_rlimits(p, &profile->rlimits))
 		return 0;
 
-	if (profile->network_allowed && kernel_supports_network) {
+	if (profile->net.allow && kernel_supports_network) {
 		size_t i;
 		if (!sd_write_array(p, "net_allowed_af", get_af_max()))
 			return 0;
 		for (i = 0; i < get_af_max(); i++) {
-		    u16 allowed = profile->network_allowed[i] &
-			~profile->deny_network[i];
+		    u16 allowed = profile->net.allow[i] &
+			~profile->net.deny[i];
 			if (!sd_write16(p, allowed))
 				return 0;
-			if (!sd_write16(p, allowed & profile->audit_network[i]))
+			if (!sd_write16(p, allowed & profile->net.audit[i]))
 				return 0;
-			if (!sd_write16(p, profile->deny_network[i] & profile->quiet_network[i]))
+			if (!sd_write16(p, profile->net.deny[i] & profile->net.quiet[i]))
 				return 0;
 		}
 		if (!sd_write_arrayend(p))
 			return 0;
-	} else if (profile->network_allowed)
+	} else if (profile->net.allow)
 		pwarn(_("profile %s network rules not enforced\n"), profile->name);
 
-	if (profile->policy_dfa && regex_type == AARE_DFA) {
+	if (profile->policy.dfa) {
 		if (!sd_write_struct(p, "policydb"))
 			return 0;
-		if (!sd_serialize_dfa(p, profile->policy_dfa, profile->policy_dfa_size))
+		if (!sd_serialize_dfa(p, profile->policy.dfa, profile->policy.size))
 			return 0;
 		if (!sd_write_structend(p))
 			return 0;
 	}
 
 	/* either have a single dfa or lists of different entry types */
-	if (regex_type == AARE_DFA) {
-		if (!sd_serialize_dfa(p, profile->dfa, profile->dfa_size))
-			return 0;
+	if (!sd_serialize_dfa(p, profile->dfa.dfa, profile->dfa.size))
+		return 0;
 
-		if (!sd_serialize_xtable(p, profile->exec_table))
-			return 0;
-	} else {
-		PERROR(_("Unknown pattern type\n"));
-		return 1;
-	}
+	if (!sd_serialize_xtable(p, profile->exec_table))
+		return 0;
 
-	if (profile->hat_table && regex_type != AARE_DFA) {
-		if (!sd_write_list(p, "hats"))
-			return 0;
-		if (load_hats(p, profile) != 0)
-			return 0;
-		if (!sd_write_listend(p))
-			return 0;
-
-	}
 	if (!sd_write_structend(p))
 		return 0;
 
 	return 1;
 }
 
-int sd_serialize_top_profile(sd_serialize *p, struct codomain *profile)
+int sd_serialize_top_profile(sd_serialize *p, Profile *profile)
 {
 	int version;
 
-	if (regex_type == AARE_DFA) {
-		/* Not yet
-		if (profile->policy_dfa)
-			version = SUBDOMAIN_INTERFACE_POLICYDB;
-		else */
-			version = SUBDOMAIN_INTERFACE_DFA_VERSION;
-	} else
-		version = SUBDOMAIN_INTERFACE_VERSION;
-
+	version = SUBDOMAIN_INTERFACE_DFA_VERSION;
 
 	if (!sd_write_name(p, "version"))
 		return 0;
@@ -711,11 +691,11 @@ int sd_serialize_top_profile(sd_serialize *p, struct codomain *profile)
 	if (!sd_write32(p, version))
 		return 0;
 
-	if (profile_namespace) {
-		if (!sd_write_string(p, profile_namespace, "namespace"))
+	if (profile_ns) {
+		if (!sd_write_string(p, profile_ns, "namespace"))
 			return 0;
-	} else if (profile->namespace) {
-		if (!sd_write_string(p, profile->namespace, "namespace"))
+	} else if (profile->ns) {
+		if (!sd_write_string(p, profile->ns, "namespace"))
 			return 0;
 	}
 
@@ -723,7 +703,7 @@ int sd_serialize_top_profile(sd_serialize *p, struct codomain *profile)
 }
 
 int cache_fd = -1;
-int sd_serialize_codomain(int option, struct codomain *cod)
+int __sd_serialize_profile(int option, Profile *prof)
 {
 	int fd = -1;
 	int error = -ENOMEM, size, wsize;
@@ -747,7 +727,7 @@ int sd_serialize_codomain(int option, struct codomain *cod)
 		if (kernel_load) fd = open(filename, O_WRONLY);
 		break;
 	case OPTION_STDOUT:
-		filename = "stdout";
+		filename = strdup("stdout");
 		fd = dup(1);
 		break;
 	case OPTION_OFILE:
@@ -768,44 +748,43 @@ int sd_serialize_codomain(int option, struct codomain *cod)
 
 	error = 0;
 
-	if (option != OPTION_STDOUT && option != OPTION_OFILE)
-		free(filename);
+	free(filename);
 
 	if (option == OPTION_REMOVE) {
 		char *name, *ns = NULL;
 		int len = 0;
 
-		if (profile_namespace) {
-			len += strlen(profile_namespace) + 2;
-			ns = profile_namespace;
-		} else if (cod->namespace) {
-			len += strlen(cod->namespace) + 2;
-			ns = cod->namespace;
+		if (profile_ns) {
+			len += strlen(profile_ns) + 2;
+			ns = profile_ns;
+		} else if (prof->ns) {
+			len += strlen(prof->ns) + 2;
+			ns = prof->ns;
 		}
-		if (cod->parent) {
-			name = malloc(strlen(cod->name) + 3 +
-				      strlen(cod->parent->name) + len);
+		if (prof->parent) {
+			name = (char *) malloc(strlen(prof->name) + 3 +
+				      strlen(prof->parent->name) + len);
 			if (!name) {
-				PERROR(_("Memory Allocation Error: Unable to remove ^%s\n"), cod->name);
+				PERROR(_("Memory Allocation Error: Unable to remove ^%s\n"), prof->name);
 				error = -errno;
 				goto exit;
 			}
 			if (ns)
 				sprintf(name, ":%s:%s//%s", ns,
-					cod->parent->name, cod->name);
+					prof->parent->name, prof->name);
 			else
-				sprintf(name, "%s//%s", cod->parent->name,
-					cod->name);
+				sprintf(name, "%s//%s", prof->parent->name,
+					prof->name);
 		} else if (ns) {
-			name = malloc(len + strlen(cod->name) + 1);
+			name = (char *) malloc(len + strlen(prof->name) + 1);
 			if (!name) {
-				PERROR(_("Memory Allocation Error: Unable to remove %s:%s."), ns, cod->name);
+				PERROR(_("Memory Allocation Error: Unable to remove %s:%s."), ns, prof->name);
 				error = -errno;
 				goto exit;
 			}
-			sprintf(name, ":%s:%s", ns, cod->name);
+			sprintf(name, ":%s:%s", ns, prof->name);
 		} else {
-			name = cod->name;
+			name = prof->name;
 		}
 		size = strlen(name) + 1;
 		if (kernel_load) {
@@ -813,7 +792,7 @@ int sd_serialize_codomain(int option, struct codomain *cod)
 			if (wsize < 0)
 				error = -errno;
 		}
-		if (cod->parent || ns)
+		if (prof->parent || ns)
 			free(name);
 	} else {
 
@@ -825,15 +804,15 @@ int sd_serialize_codomain(int option, struct codomain *cod)
 			goto exit;
 		}
 
-		if (!sd_serialize_top_profile(work_area, cod)) {
+		if (!sd_serialize_top_profile(work_area, prof)) {
 			close(fd);
 			free_sd_serial(work_area);
 			PERROR(_("unable to serialize profile %s\n"),
-			       cod->name);
+			       prof->name);
 			goto exit;
 		}
 
-		size = work_area->pos - work_area->buffer;
+		size = (long) (work_area->pos) - (long)(work_area->buffer);
 		if (kernel_load || option == OPTION_STDOUT || option == OPTION_OFILE) {
 			wsize = write(fd, work_area->buffer, size);
 			if (wsize < 0) {
@@ -859,8 +838,8 @@ int sd_serialize_codomain(int option, struct codomain *cod)
 
 	close(fd);
 
-	if (cod->hat_table && regex_type == AARE_DFA && option != OPTION_REMOVE) {
-		if (load_flattened_hats(cod) != 0)
+	if (!prof->hat_table.empty() && option != OPTION_REMOVE) {
+		if (load_flattened_hats(prof, option) == 0)
 			return 0;
 	}
 
