@@ -2,7 +2,7 @@
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *   NOVELL (All rights reserved)
  *
- *   Copyright (c) 2010 - 2012
+ *   Copyright (c) 2010 - 2013
  *   Canonical Ltd. (All rights reserved)
  *
  *   This program is free software; you can redistribute it and/or
@@ -24,10 +24,10 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <stddef.h>
 #include <getopt.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <mntent.h>
 #include <libintl.h>
 #include <locale.h>
 #include <dirent.h>
@@ -41,7 +41,9 @@
 #include <sys/sysctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/apparmor.h>
 
+#include "lib.h"
 #include "parser.h"
 #include "parser_version.h"
 #include "parser_include.h"
@@ -62,7 +64,6 @@
 const char *parser_title	= "AppArmor parser";
 const char *parser_copyright	= "Copyright (C) 1999-2008 Novell Inc.\nCopyright 2009-2012 Canonical Ltd.";
 
-char *progname;
 int opt_force_complain = 0;
 int binary_input = 0;
 int dump_vars = 0;
@@ -71,11 +72,14 @@ int show_cache = 0;
 int skip_cache = 0;
 int skip_read_cache = 0;
 int write_cache = 0;
+int cond_clear_cache = 1;		/* only applies if write is set */
+int force_clear_cache = 0;		/* force clearing regargless of state */
+int create_cache_dir = 0;		/* create the cache dir if missing? */
 int preprocess_only = 0;
 int skip_mode_force = 0;
 struct timespec mru_tstamp;
 
-#define FLAGS_STRING_SIZE 1024
+#define FLAGS_STRING_SIZE 8192
 char *match_string = NULL;
 char *flags_string = NULL;
 char *cacheloc = NULL;
@@ -83,11 +87,13 @@ char *cacheloc = NULL;
 /* per-profile settings */
 int force_complain = 0;
 
+/* Make sure to update BOTH the short and long_options */
+static const char *short_options = "adf:h::rRVvI:b:BCD:NSm:qQn:XKTWkL:O:po:";
 struct option long_options[] = {
 	{"add", 		0, 0, 'a'},
 	{"binary",		0, 0, 'B'},
 	{"base",		1, 0, 'b'},
-	{"subdomainfs",		0, 0, 'f'},
+	{"subdomainfs",		1, 0, 'f'},
 	{"help",		2, 0, 'h'},
 	{"replace",		0, 0, 'r'},
 	{"reload",		0, 0, 'r'},	/* undocumented reload option == replace */
@@ -109,6 +115,9 @@ struct option long_options[] = {
 	{"skip-read-cache",	0, 0, 'T'},
 	{"write-cache",		0, 0, 'W'},
 	{"show-cache",		0, 0, 'k'},
+	{"skip-bad-cache",	0, 0, 129},	/* no short option */
+	{"purge-cache",		0, 0, 130},	/* no short option */
+	{"create-cache-dir",	0, 0, 131},	/* no short option */
 	{"cache-loc",		1, 0, 'L'},
 	{"debug",		0, 0, 'd'},
 	{"dump",		1, 0, 'D'},
@@ -127,7 +136,7 @@ static void display_version(void)
 	       parser_copyright);
 }
 
-static void display_usage(char *command)
+static void display_usage(const char *command)
 {
 	display_version();
 	printf("\nUsage: %s [options] [profile]\n\n"
@@ -151,6 +160,9 @@ static void display_usage(char *command)
 	       "-K, --skip-cache	Do not attempt to load or save cached profiles\n"
 	       "-T, --skip-read-cache	Do not attempt to load cached profiles\n"
 	       "-W, --write-cache	Save cached profile (force with -T)\n"
+	       "    --skip-bad-cache	Don't clear cache if out of sync\n"
+	       "    --purge-cache	Clear cache regardless of its state\n"
+	       "    --create-cache-dir	Create the cache dir if missing\n"
 	       "-L, --cache-loc n	Set the location of the profile cache\n"
 	       "-q, --quiet		Don't emit warnings\n"
 	       "-v, --verbose		Show profile names as they load\n"
@@ -184,10 +196,10 @@ optflag_table_t dumpflag_table[] = {
 	  DFA_DUMP_SIMPLE_TREE },
 	{ 1, "stats", "Dump all compile stats",
 	  DFA_DUMP_TREE_STATS | DFA_DUMP_STATS | DFA_DUMP_TRANS_STATS |
-	  DFA_DUMP_EQUIV_STATS },
+	  DFA_DUMP_EQUIV_STATS | DFA_DUMP_DIFF_STATS },
 	{ 1, "progress", "Dump progress for all compile phases",
 	  DFA_DUMP_PROGRESS | DFA_DUMP_STATS | DFA_DUMP_TRANS_PROGRESS |
-	  DFA_DUMP_TRANS_STATS },
+	  DFA_DUMP_TRANS_STATS | DFA_DUMP_DIFF_PROGRESS | DFA_DUMP_DIFF_STATS },
 	{ 1, "dfa-progress", "Dump dfa creation as in progress",
 	  DFA_DUMP_PROGRESS | DFA_DUMP_STATS },
 	{ 1, "dfa-stats", "Dump dfa creation stats", DFA_DUMP_STATS },
@@ -212,13 +224,20 @@ optflag_table_t dumpflag_table[] = {
 	{ 1, "equiv-stats", "Dump equivance class stats",
 	  DFA_DUMP_EQUIV_STATS },
 	{ 1, "equiv", "Dump equivance class", DFA_DUMP_EQUIV },
+	{ 1, "diff-encode", "Dump differential encoding",
+	  DFA_DUMP_DIFF_ENCODE },
+	{ 1, "diff-stats", "Dump differential encoding stats",
+	  DFA_DUMP_DIFF_STATS },
+	{ 1, "diff-progress", "Dump progress of differential encoding",
+	  DFA_DUMP_DIFF_PROGRESS | DFA_DUMP_DIFF_STATS },
 	{ 0, NULL, NULL, 0 },
 };
 
 optflag_table_t optflag_table[] = {
 	{ 2, "0", "no optimizations",
 	  DFA_CONTROL_TREE_NORMAL | DFA_CONTROL_TREE_SIMPLE |
-	  DFA_CONTROL_MINIMIZE | DFA_CONTROL_REMOVE_UNREACHABLE
+	  DFA_CONTROL_MINIMIZE | DFA_CONTROL_REMOVE_UNREACHABLE |
+	  DFA_CONTROL_DIFF_ENCODE
 	},
 	{ 1, "equiv", "use equivalent classes", DFA_CONTROL_EQUIV },
 	{ 1, "expr-normalize", "expression tree normalization",
@@ -230,8 +249,6 @@ optflag_table_t optflag_table[] = {
 	{ 2, "expr-right-simplify", "right simplification first",
 	  DFA_CONTROL_TREE_LEFT },
 	{ 1, "minimize", "dfa state minimization", DFA_CONTROL_MINIMIZE },
-	{ 1, "hash-trans", "minimization - hash transitions during setup",
-	  DFA_CONTROL_MINIMIZE_HASH_TRANS },
 	{ 1, "filter-deny", "filter out deny information from final dfa",
 	  DFA_CONTROL_FILTER_DENY },
 	{ 1, "remove-unreachable", "dfa unreachable state removal",
@@ -241,6 +258,8 @@ optflag_table_t optflag_table[] = {
 	  DFA_CONTROL_TRANS_HIGH },
 	{ 2, "compress-fast", "do faster dfa transition table compression",
 	  DFA_CONTROL_TRANS_HIGH },
+	{ 1, "diff-encode", "Differentially encode transitions",
+	  DFA_CONTROL_DIFF_ENCODE },
 	{ 0, NULL, NULL, 0 },
 };
 
@@ -288,7 +307,7 @@ static int handle_flag_table(optflag_table_t *table, const char *optarg,
 	return 0;
 }
 
-static void display_dump(char *command)
+static void display_dump(const char *command)
 {
 	display_version();
 	printf("\n%s: --dump [Option]\n\n"
@@ -300,7 +319,7 @@ static void display_dump(char *command)
 	print_flag_table(dumpflag_table);
 }
 
-static void display_optimize(char *command)
+static void display_optimize(const char *command)
 {
 	display_version();
 	printf("\n%s: -O [Option]\n\n"
@@ -449,8 +468,10 @@ static int process_arg(int c, char *optarg)
 		skip_cache = 1;
 		break;
 	case 'N':
+		count++;
 		names_only = 1;
 		skip_cache = 1;
+		kernel_load = 0;
 		break;
 	case 'S':
 		count++;
@@ -510,7 +531,7 @@ static int process_arg(int c, char *optarg)
 		conf_quiet = 0;
 		break;
 	case 'n':
-		profile_namespace = strdup(optarg);
+		profile_ns = strdup(optarg);
 		break;
 	case 'X':
 		read_implies_exec = 1;
@@ -527,6 +548,15 @@ static int process_arg(int c, char *optarg)
 	case 'T':
 		skip_read_cache = 1;
 		break;
+	case 129:
+		cond_clear_cache = 0;
+		break;
+	case 130:
+		force_clear_cache = 1;
+		break;
+	case 131:
+		create_cache_dir = 1;
+		break;
 	case 'L':
 		cacheloc = strdup(optarg);
 		break;
@@ -542,7 +572,7 @@ static int process_arg(int c, char *optarg)
 		break;
 	default:
 		display_usage(progname);
-		exit(0);
+		exit(1);
 		break;
 	}
 
@@ -555,7 +585,7 @@ static int process_args(int argc, char *argv[])
 	int count = 0;
 	option = OPTION_ADD;
 
-	while ((c = getopt_long(argc, argv, "adf:h::rRVvI:b:BCD:NSm:qQn:XKTWkO:po:", long_options, &o)) != -1)
+	while ((c = getopt_long(argc, argv, short_options, long_options, &o)) != -1)
 	{
 		count += process_arg(c, optarg);
 	}
@@ -583,75 +613,27 @@ static int process_config_file(const char *name)
 
 	while ((c = getopt_long_file(f, long_options, &optarg, &o)) != -1)
 		process_arg(c, optarg);
+	fclose(f);
 	return 1;
 }
 
-static inline char *try_subdomainfs_mountpoint(const char *mntpnt,
-					       const char *path)
-{
-	char *proposed_base = NULL;
-	char *retval = NULL;
-	struct stat buf;
-
-	if (asprintf(&proposed_base, "%s%s", mntpnt, path)<0 || !proposed_base) {
-		PERROR(_("%s: Could not allocate memory for subdomainbase mount point\n"),
-		       progname);
-		exit(ENOMEM);
-	}
-	if (stat(proposed_base, &buf) == 0) {
-		retval = proposed_base;
-	} else {
-		free(proposed_base);
-	}
-	return retval;
-}
 
 int find_subdomainfs_mountpoint(void)
 {
-	FILE *mntfile;
-	struct mntent *mntpt;
-
-	if ((mntfile = setmntent(MOUNTED_FS, "r"))) {
-		while ((mntpt = getmntent(mntfile))) {
-			char *proposed = NULL;
-			if (strcmp(mntpt->mnt_type, "securityfs") == 0) {
-				proposed = try_subdomainfs_mountpoint(mntpt->mnt_dir, "/" MODULE_NAME);
-				if (proposed != NULL) {
-					subdomainbase = proposed;
-					break;
-				}
-				proposed = try_subdomainfs_mountpoint(mntpt->mnt_dir, "/" OLD_MODULE_NAME);
-				if (proposed != NULL) {
-					subdomainbase = proposed;
-					break;
-				}
-			}
-			if (strcmp(mntpt->mnt_type, "subdomainfs") == 0) {
-				proposed = try_subdomainfs_mountpoint(mntpt->mnt_dir, "");
-				if (proposed != NULL) {
-					subdomainbase = proposed;
-					break;
-				}
-			}
-		}
-		endmntent(mntfile);
-	}
-
-	if (!subdomainbase) {
+	if (aa_find_mountpoint(&subdomainbase) == -1) {
 		struct stat buf;
 		if (stat(DEFAULT_APPARMORFS, &buf) == -1) {
 		PERROR(_("Warning: unable to find a suitable fs in %s, is it "
 			 "mounted?\nUse --subdomainfs to override.\n"),
 		       MOUNTED_FS);
 		} else {
-			subdomainbase = DEFAULT_APPARMORFS;
+			subdomainbase = strdup(DEFAULT_APPARMORFS);
 		}
 	}
 
 	return (subdomainbase == NULL);
 }
 
-
 int have_enough_privilege(void)
 {
 	uid_t uid, euid;
@@ -692,89 +674,70 @@ char *snprintf_buffer(char *buf, char *pos, ssize_t size, const char *fmt, ...)
 	return pos + i;
 }
 
+struct features_struct {
+	char **buffer;
+	int size;
+	char *pos;
+};
+
+static int features_dir_cb(DIR *dir, const char *name, struct stat *st,
+			   void *data)
+{
+	struct features_struct *fst = (struct features_struct *) data;
+
+	/* skip dot files and files with no name */
+	if (*name == '.' || !strlen(name))
+		return 0;
+
+	fst->pos = snprintf_buffer(*fst->buffer, fst->pos, fst->size, "%s {", name);
+
+	if (S_ISREG(st->st_mode)) {
+		int len, file;
+		int remaining = fst->size - (fst->pos - *fst->buffer);
+		if (!(file = openat(dirfd(dir), name, O_RDONLY))) {
+			PDEBUG("Could not open '%s'", name);
+			return -1;
+		}
+		PDEBUG("Opened features \"%s\"\n", name);
+		if (st->st_size > remaining) {
+			PDEBUG("Feature buffer full.");
+			return -1;
+		}
+
+		do {
+			len = read(file, fst->pos, remaining);
+			if (len > 0) {
+				remaining -= len;
+				fst->pos += len;
+				*fst->pos = 0;
+			}
+		} while (len > 0);
+		if (len < 0) {
+			PDEBUG("Error reading feature file '%s'\n", name);
+			return -1;
+		}
+		close(file);
+	} else if (S_ISDIR(st->st_mode)) {
+		if (dirat_for_each(dir, name, fst, features_dir_cb))
+			return -1;
+	}
+
+	fst->pos = snprintf_buffer(*fst->buffer, fst->pos, fst->size, "}\n");
+
+	return 0;
+}
+
 static char *handle_features_dir(const char *filename, char **buffer, int size,
 				 char *pos)
 {
-	DIR *dir = NULL;
-	char *dirent_path = NULL;
-	struct dirent *dirent;
-	struct stat my_stat;
-	int len;
+	struct features_struct fst = { buffer, size, pos };
 
-	PDEBUG("Opened features directory \"%s\"\n", filename);
-	if (!(dir = opendir(filename))) {
-		PDEBUG("opendir failed '%s'", filename);
+	if (dirat_for_each(NULL, filename, &fst, features_dir_cb)) {
+		PDEBUG("Failed evaluating .features\n");
 		exit(1);
 	}
 
-	while ((dirent = readdir(dir)) != NULL) {
-		int name_len;
-		/* skip dotfiles silently. */
-		if (dirent->d_name[0] == '.')
-			continue;
-
-		if (dirent_path)
-			free(dirent_path);
-		if (asprintf(&dirent_path, "%s/%s", filename, dirent->d_name) < 0)
-		{
-			PERROR(_("Memory allocation error."));
-			exit(1);
-		}
-
-		name_len = strlen(dirent->d_name);
-		if (!name_len)
-			continue;
-
-		if (stat(dirent_path, &my_stat)) {
-			PERROR(_("stat failed for '%s'"), dirent_path);
-			exit(1);
-		}
-
-		pos = snprintf_buffer(*buffer, pos, size, "%s {", dirent->d_name);
-		if (S_ISREG(my_stat.st_mode)) {
-			int file;
-			int remaining = size - (pos - *buffer);
-			if (!(file = open(dirent_path, O_RDONLY))) {
-				PDEBUG("Could not open '%s' in '%s'", dirent_path, filename);
-				exit(1);
-				break;
-			}
-			PDEBUG("Opened features \"%s\" in \"%s\"\n", dirent_path, filename);
-			if (my_stat.st_size > remaining) {
-				PERROR(_("Feature buffer full."));
-				exit(1);
-			}
-
-			do {
-				len = read(file, pos, remaining);
-				if (len > 0) {
-					remaining -= len;
-					pos += len;
-					*pos = 0;
-				}
-			} while (len > 0);
-			if (len < 0) {
-				PDEBUG("Error reading feature file '%s'\n",
-				       dirent_path);
-				exit(1);
-			}
-			close(file);
-
-		} else if (S_ISDIR(my_stat.st_mode)) {
-			pos = handle_features_dir(dirent_path, buffer, size,
-						  pos);
-			if (!pos)
-				break;
-
-		}
-
-		pos = snprintf_buffer(*buffer, pos, size, "}\n");
-	}
-	if (dirent_path)
-		free(dirent_path);
-	closedir(dir);
-
-	return pos;
+	return fst.pos;
 }
 
 /* match_string == NULL --> no match_string available
@@ -794,10 +757,9 @@ static void get_match_string(void) {
 
 	if (S_ISDIR(stat_file.st_mode)) {
 		/* if we have a features directory default to */
-		regex_type = AARE_DFA;
 		perms_create = 1;
 
-		flags_string = malloc(FLAGS_STRING_SIZE);
+		flags_string = (char *) malloc(FLAGS_STRING_SIZE);
 		handle_features_dir(FLAGS_FILE, &flags_string, FLAGS_STRING_SIZE, flags_string);
 		if (strstr(flags_string, "network"))
 			kernel_supports_network = 1;
@@ -805,6 +767,8 @@ static void get_match_string(void) {
 			kernel_supports_network = 0;
 		if (strstr(flags_string, "mount"))
 			kernel_supports_mount = 1;
+		if (strstr(flags_string, "dbus"))
+			kernel_supports_dbus = 1;
 		return;
 	}
 
@@ -812,7 +776,7 @@ static void get_match_string(void) {
 	if (!ms)
 		goto out;
 
-	match_string = malloc(1000);
+	match_string = (char *) malloc(1000);
 	if (!match_string) {
 		goto out;
 	}
@@ -824,16 +788,9 @@ static void get_match_string(void) {
 
 out:
 	if (match_string) {
-		if (strstr(match_string, AADFA))
-			regex_type = AARE_DFA;
-
 		if (strstr(match_string, " perms=c"))
 			perms_create = 1;
 	} else {
-		/* no match string default to 2.6.36 version which doesn't
-		 * have a match string
-		 */
-		regex_type = AARE_DFA;
 		perms_create = 1;
 		kernel_supports_network = 0;
 	}
@@ -843,7 +800,7 @@ out:
 	return;
 }
 
-static void get_flags_string(char **flags, char *flags_file) {
+static void get_flags_string(char **flags, const char *flags_file) {
 	char *pos;
 	FILE *f = NULL;
 	size_t size;
@@ -856,7 +813,7 @@ static void get_flags_string(char **flags, char *flags_file) {
 	if (!f)
 		return;
 
-	*flags = malloc(FLAGS_STRING_SIZE);
+	*flags = (char *) malloc(FLAGS_STRING_SIZE);
 	if (!*flags)
 		goto fail;
 
@@ -866,13 +823,6 @@ static void get_flags_string(char **flags, char *flags_file) {
 	(*flags)[size] = 0;
 
 	fclose(f);
-	pos = strstr(*flags, "change_hat=");
-	if (pos) {
-		if (strncmp(pos, "change_hat=1.4", 14) == 0)
-			flag_changehat_version = FLAG_CHANGEHAT_1_4;
-//fprintf(stderr, "flags string: %s\n", flags_string);
-//fprintf(stderr, "changehat %d\n", flag_changehat_version);
-	}
 	return;
 
 fail:
@@ -883,7 +833,7 @@ fail:
 	return;
 }
 
-int process_binary(int option, char *profilename)
+int process_binary(int option, const char *profilename)
 {
 	char *buffer = NULL;
 	int retval = 0, size = 0, asize = 0, rsize;
@@ -903,7 +853,7 @@ int process_binary(int option, char *profilename)
 
 	do {
 		if (asize - size == 0) {
-			buffer = realloc(buffer, chunksize);
+		  buffer = (char *) realloc(buffer, chunksize);
 			asize = chunksize;
 			chunksize <<= 1;
 			if (!buffer) {
@@ -944,7 +894,7 @@ int process_binary(int option, char *profilename)
 	return retval;
 }
 
-void reset_parser(char *filename)
+void reset_parser(const char *filename)
 {
 	memset(&mru_tstamp, 0, sizeof(mru_tstamp));
 	free_aliases();
@@ -989,13 +939,13 @@ void update_mru_tstamp(FILE *file)
 		mru_tstamp = stat_file.st_ctim;
 }
 
-int process_profile(int option, char *profilename)
+int process_profile(int option, const char *profilename)
 {
 	struct stat stat_bin;
 	int retval = 0;
 	char * cachename = NULL;
 	char * cachetemp = NULL;
-	char *basename = NULL;
+	const char *basename = NULL;
 	FILE *cmd;
 
 	/* per-profile states */
@@ -1060,7 +1010,7 @@ int process_profile(int option, char *profilename)
 	 * TODO: Add support for embedded namespace defines if they aren't
 	 *       removed from the language.
 	 */
-	if (profile_namespace)
+	if (profile_ns)
 		skip_cache = 1;
 
 	/* Do secondary test to see if cached binary profile is good,
@@ -1071,13 +1021,7 @@ int process_profile(int option, char *profilename)
 	 */
 	if ((profilename && option != OPTION_REMOVE) && !force_complain &&
 	    !skip_cache) {
-		if (cacheloc) {
-			cachename = strdup(cacheloc);
-			if (!cachename) {
-				PERROR(_("Memory allocation error."));
-				exit(1);
-			}
-		} else if (asprintf(&cachename, "%s/%s/%s", basedir, "cache", basename)<0) {
+		if (asprintf(&cachename, "%s/%s", cacheloc, basename)<0) {
 			PERROR(_("Memory allocation error."));
 			exit(1);
 		}
@@ -1092,7 +1036,7 @@ int process_profile(int option, char *profilename)
 		}
 		if (write_cache) {
 			/* Otherwise, set up to save a cached copy */
-			if (asprintf(&cachetemp, "%s/%s/%s-XXXXXX", basedir, "cache", basename)<0) {
+			if (asprintf(&cachetemp, "%s-XXXXXX", cachename)<0) {
 				perror("asprintf");
 				exit(1);
 			}
@@ -1150,8 +1094,11 @@ out:
 		}
 
 		if (useable_cache) {
-			rename(cachetemp, cachename);
-			if (show_cache)
+			if (rename(cachetemp, cachename) < 0) {
+				pwarn("Warning failed to write cache: %s\n", cachename);
+				unlink(cachetemp);
+			}
+			else if (show_cache)
 				PERROR("Wrote cache: %s\n", cachename);
 		}
 		else {
@@ -1165,6 +1112,99 @@ out:
 	return retval;
 }
 
+/* data - name of parent dir */
+static int profile_dir_cb(__unused DIR *dir, const char *name, struct stat *st,
+			  void *data)
+{
+	int rc = 0;
+
+	if (!S_ISDIR(st->st_mode) && !is_blacklisted(name, NULL)) {
+		const char *dirname = (const char *)data;
+		char *path;
+		if (asprintf(&path, "%s/%s", dirname, name) < 0)
+			PERROR(_("Out of memory"));
+		rc = process_profile(option, path);
+		free(path);
+	}
+	return rc;
+}
+
+/* data - name of parent dir */
+static int binary_dir_cb(__unused DIR *dir, const char *name, struct stat *st,
+			 void *data)
+{
+	int rc = 0;
+
+	if (!S_ISDIR(st->st_mode) && !is_blacklisted(name, NULL)) {
+		const char *dirname = (const char *)data;
+		char *path;
+		if (asprintf(&path, "%s/%s", dirname, name) < 0)
+			PERROR(_("Out of memory"));
+		rc = process_binary(option, path);
+		free(path);
+	}
+	return rc;
+}
+
+static int clear_cache_cb(DIR *dir, const char *path, struct stat *st,
+			  __unused void *data)
+{
+	/* remove regular files */
+	if (S_ISREG(st->st_mode))
+		return unlinkat(dirfd(dir), path, 0);
+
+	/* do nothing with other file types */
+	return 0;
+}
+
+static int clear_cache_files(const char *path)
+{
+	return dirat_for_each(NULL, path, NULL, clear_cache_cb);
+}
+
+static int create_cache(const char *cachedir, const char *path,
+			const char *features)
+{
+	struct stat stat_file;
+	FILE * f = NULL;
+
+	if (clear_cache_files(cacheloc) != 0)
+		goto error;
+
+create_file:
+	f = fopen(path, "w");
+	if (f) {
+		if (fwrite(features, strlen(features), 1, f) != 1 )
+			goto error;
+
+		fclose(f);
+
+
+		return 0;
+	}
+
+error:
+	/* does the dir exist? */
+	if (stat(cachedir, &stat_file) == -1 && create_cache_dir) {
+		if (mkdir(cachedir, 0700) == 0)
+			goto create_file;
+		if (show_cache)
+			PERROR(_("Can't create cache directory: %s\n"), cachedir);
+	} else if (!S_ISDIR(stat_file.st_mode)) {
+		if (show_cache)
+			PERROR(_("File in cache directory location: %s\n"), cachedir);
+	} else {
+		if (show_cache)
+			PERROR(_("Can't update cache directory: %s\n"), cachedir);
+	}
+
+	if (show_cache)
+		PERROR("Cache write disabled: cannot create %s\n", path);
+	write_cache = 0;
+
+	return -1;
+}
+
 static void setup_flags(void)
 {
 	char *cache_features_path = NULL;
@@ -1182,7 +1222,12 @@ static void setup_flags(void)
 		write_cache = 0;
 		skip_read_cache = 1;
 		return;
-	}
+	} else if (strstr(flags_string, "network"))
+		kernel_supports_network = 1;
+	else
+		kernel_supports_network = 0;
+
+
 
 	/*
          * Deal with cache directory versioning:
@@ -1190,38 +1235,29 @@ static void setup_flags(void)
          *  - If cache/.features exists, and does not match flags_string,
          *    force cache reading/writing off.
          */
-	if (asprintf(&cache_features_path, "%s/cache/.features", basedir) == -1) {
-		perror("asprintf");
+	if (asprintf(&cache_features_path, "%s/.features", cacheloc) == -1) {
+		PERROR(_("Memory allocation error."));
 		exit(1);
 	}
 
 	get_flags_string(&cache_flags, cache_features_path);
 	if (cache_flags) {
 		if (strcmp(flags_string, cache_flags) != 0) {
-			if (show_cache) PERROR("Cache read/write disabled: %s does not match %s\n", FLAGS_FILE, cache_features_path);
-			write_cache = 0;
-			skip_read_cache = 1;
+			if (write_cache && cond_clear_cache) {
+				if (create_cache(cacheloc, cache_features_path,
+						 flags_string))
+					skip_read_cache = 1;
+			} else {
+				if (show_cache)
+					PERROR("Cache read/write disabled: %s does not match %s\n", FLAGS_FILE, cache_features_path);
+				write_cache = 0;
+				skip_read_cache = 1;
+			}
 		}
 		free(cache_flags);
 		cache_flags = NULL;
-	}
-	else if (write_cache) {
-		FILE * f = NULL;
-		int failure = 0;
-
-		f = fopen(cache_features_path, "w");
-		if (!f) failure = 1;
-		else {
-			if (fwrite(flags_string, strlen(flags_string), 1, f) != 1 ) {
-				failure = 1;
-			}
-			if (fclose(f) != 0) failure = 1;
-		}
-
-		if (failure) {
-			if (show_cache) PERROR("Cache write disabled: cannot write to %s\n", cache_features_path);
-			write_cache = 0;
-		}
+	} else if (write_cache) {
+		create_cache(cacheloc, cache_features_path, flags_string);
 	}
 
 	free(cache_features_path);
@@ -1251,6 +1287,17 @@ int main(int argc, char *argv[])
 		return retval;
 	}
 
+	/* create the cacheloc once and use it everywhere */
+	if (!cacheloc) {
+		if (asprintf(&cacheloc, "%s/cache", basedir) == -1) {
+			PERROR(_("Memory allocation error."));
+			exit(1);
+		}
+	}
+
+	if (force_clear_cache) 
+		exit(clear_cache_files(cacheloc));
+
 	/* Check to make sure there is an interface to load policy */
 	if (!(UNPRIVILEGED_OPS) && (subdomainbase == NULL) &&
 	    (retval = find_subdomainfs_mountpoint())) {
@@ -1263,6 +1310,8 @@ int main(int argc, char *argv[])
 
 	retval = 0;
 	for (i = optind; retval == 0 && i <= argc; i++) {
+		struct stat stat_file;
+
 		if (i < argc && !(profilename = strdup(argv[i]))) {
 			perror("strdup");
 			return -1;
@@ -1271,7 +1320,21 @@ int main(int argc, char *argv[])
 		if (i == argc && optind != argc)
 			continue;
 
-		if (binary_input) {
+		if (profilename && stat(profilename, &stat_file) == -1) {
+			PERROR("File %s not found, skipping...\n", profilename);
+			continue;
+		}
+
+		if (profilename && S_ISDIR(stat_file.st_mode)) {
+			int (*cb)(DIR *dir, const char *name, struct stat *st,
+				  void *data);
+			cb = binary_input ? binary_dir_cb : profile_dir_cb;
+			if (dirat_for_each(NULL, profilename, profilename, cb)) {
+				PDEBUG("Failed loading profiles from %s\n",
+				       profilename);
+				exit(1);
+			}
+		} else if (binary_input) {
 			retval = process_binary(option, profilename);
 		} else {
 			retval = process_profile(option, profilename);
