@@ -2,6 +2,9 @@
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *   NOVELL (All rights reserved)
  *
+ *   Copyright (c) 2013
+ *   Canonical Ltd. (All rights reserved)
+ *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of version 2 of the GNU General Public
  *   License published by the Free Software Foundation.
@@ -34,10 +37,13 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <sys/apparmor.h>
 
 #include "parser.h"
+#include "profile.h"
 #include "parser_yacc.h"
 #include "mount.h"
+#include "dbus.h"
 
 /* #define DEBUG */
 #ifdef DEBUG
@@ -49,8 +55,57 @@
 #endif
 #define NPDEBUG(fmt, args...)	/* Do nothing */
 
+struct ignored_suffix_t {
+	const char * text;
+	int len;
+	int silent;
+};
+
+static struct ignored_suffix_t ignored_suffixes[] = {
+	/* Debian packging files, which are in flux during install
+           should be silently ignored. */
+	{ ".dpkg-new", 9, 1 },
+	{ ".dpkg-old", 9, 1 },
+	{ ".dpkg-dist", 10, 1 },
+	{ ".dpkg-bak", 9, 1 },
+	/* RPM packaging files have traditionally not been silently
+           ignored */
+	{ ".rpmnew", 7, 0 },
+	{ ".rpmsave", 8, 0 },
+	/* patch file backups/conflicts */
+	{ ".orig", 5, 0 },
+	{ ".rej", 4, 0 },
+	/* Backup files should be mentioned */
+	{ "~", 1, 0 },
+	{ NULL, 0, 0 }
+};
+
+int is_blacklisted(const char *name, const char *path)
+{
+	int name_len;
+	struct ignored_suffix_t *suffix;
+
+	/* skip dot files and files with no name */
+	if (*name == '.' || !strlen(name))
+		return 1;
+
+	name_len = strlen(name);
+	/* skip blacklisted suffixes */
+	for (suffix = ignored_suffixes; suffix->text; suffix++) {
+		char *found;
+		if ( (found = strstr((char *) name, suffix->text)) &&
+		     found - name + suffix->len == name_len ) {
+			if (!suffix->silent)
+				PERROR("Ignoring: '%s'\n", path ? path : name);
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 struct keyword_table {
-	char *keyword;
+	const char *keyword;
 	int token;
 };
 
@@ -73,6 +128,7 @@ static struct keyword_table keyword_table[] = {
 	{"subset",		TOK_SUBSET},
 	{"audit",		TOK_AUDIT},
 	{"deny",		TOK_DENY},
+	{"allow",		TOK_ALLOW},
 	{"set",			TOK_SET},
 	{"rlimit",		TOK_RLIMIT},
 	{"alias",		TOK_ALIAS},
@@ -85,6 +141,15 @@ static struct keyword_table keyword_table[] = {
 	{"unmount",		TOK_UMOUNT},
 	{"pivot_root",		TOK_PIVOTROOT},
 	{"in",			TOK_IN},
+	{"dbus",		TOK_DBUS},
+	{"send",                TOK_SEND},
+	{"receive",             TOK_RECEIVE},
+	{"bind",                TOK_BIND},
+	{"read",                TOK_READ},
+	{"write",               TOK_WRITE},
+	{"eavesdrop",		TOK_EAVESDROP},
+	{"peer",		TOK_PEER},
+
 	/* terminate */
 	{NULL, 0}
 };
@@ -109,6 +174,9 @@ static struct keyword_table rlimit_table[] = {
 #endif
 #ifdef RLIMIT_RTPRIO
 	{"rtprio",		RLIMIT_RTPRIO},
+#endif
+#ifdef RLIMIT_RTTIME
+	{"rttime",		RLIMIT_RTTIME},
 #endif
 	/* terminate */
 	{NULL, 0}
@@ -159,11 +227,11 @@ int get_rlimit(const char *name)
 }
 
 struct network_tuple {
-	char *family_name;
+	const char *family_name;
 	unsigned int family;
-	char *type_name;
+	const char *type_name;
 	unsigned int type;
-	char *protocol_name;
+	const char *protocol_name;
 	unsigned int protocol;
 };
 
@@ -184,7 +252,13 @@ struct network_tuple {
 
 /* used by af_name.h to auto generate table entries for "name", AF_NAME
  * pair */
-#define AA_GEN_NET_ENT(name, AF) {name, AF, "stream", SOCK_STREAM, "", 0xffffff}, {name, AF, "dgram", SOCK_DGRAM, "", 0xffffff}, {name, AF, "seqpacket", SOCK_SEQPACKET, "", 0xffffff}, {name, AF, "rdm", SOCK_RDM, "", 0xffffff}, {name, AF, "raw", SOCK_RAW, "", 0xffffff}, {name, AF, "packet", SOCK_PACKET, "", 0xffffff},
+#define AA_GEN_NET_ENT(name, AF) \
+	{name, AF, "stream",    SOCK_STREAM,    "", 0xffffff}, \
+	{name, AF, "dgram",     SOCK_DGRAM,     "", 0xffffff}, \
+	{name, AF, "seqpacket", SOCK_SEQPACKET, "", 0xffffff}, \
+	{name, AF, "rdm",       SOCK_RDM,       "", 0xffffff}, \
+	{name, AF, "raw",       SOCK_RAW,       "", 0xffffff}, \
+	{name, AF, "packet",    SOCK_PACKET,    "", 0xffffff},
 /*FIXME: missing {name, AF, "dccp", SOCK_DCCP, "", 0xfffffff}, */
 
 static struct network_tuple network_mappings[] = {
@@ -242,11 +316,11 @@ static size_t kernel_af_max(void) {
 	if (!fd)
 		/* fall back to default provided during build */
 		return 0;
-	res = read(fd, &buffer, sizeof(buffer));
+	res = read(fd, &buffer, sizeof(buffer) - 1);
 	close(fd);
-	if (!res)
+	if (res <= 0)
 		return 0;
-	buffer[sizeof(buffer)-1] = '\0';
+	buffer[res] = '\0';
 	res = sscanf(buffer, "2.6.%d", &major);
 	if (res != 1)
 		return 0;
@@ -318,7 +392,7 @@ struct aa_network_entry *new_network_ent(unsigned int family,
 					 unsigned int protocol)
 {
 	struct aa_network_entry *new_entry;
-	new_entry = calloc(1, sizeof(struct aa_network_entry));
+	new_entry = (struct aa_network_entry *) calloc(1, sizeof(struct aa_network_entry));
 	if (new_entry) {
 		new_entry->family = family;
 		new_entry->type = type;
@@ -367,7 +441,7 @@ struct aa_network_entry *network_entry(const char *family, const char *type,
 	return entry;
 };
 
-char *processunquoted(char *string, int len)
+char *processunquoted(const char *string, int len)
 {
 	char *tmp, *s;
 	int l;
@@ -403,7 +477,7 @@ char *processunquoted(char *string, int len)
 	return tmp;
 }
 
-char *processid(char *string, int len)
+char *processid(const char *string, int len)
 {
 	/* lexer should never call this fn if len <= 0 */
 	assert(len > 0);
@@ -416,7 +490,7 @@ char *processid(char *string, int len)
 /* rewrite a quoted string substituting escaped characters for the
  * real thing.  Strip the quotes around the string */
 
-char *processquoted(char *string, int len)
+char *processquoted(const char *string, int len)
 {
 	char *tmp, *s;
 	int l;
@@ -546,13 +620,13 @@ static int parse_sub_mode(const char *str_mode, const char *mode_desc __unused)
 
 	p = str_mode;
 	while (*p) {
-		char this = *p;
+		char thisc = *p;
 		char next = *(p + 1);
 		char lower;
 		int tmode = 0;
 
 reeval:
-		switch (this) {
+		switch (thisc) {
 		case COD_READ_CHAR:
 			if (read_implies_exec) {
 				PDEBUG("Parsing mode: found %s READ imply X\n", mode_desc);
@@ -610,7 +684,7 @@ reeval:
 			PDEBUG("Parsing mode: found UNCONFINED\n");
 			if (IS_DIFF_QUAL(mode, tmode)) {
 				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"),
-					this);
+					thisc);
 			} else {
 				if (next != tolower(next))
 					warn_uppercase();
@@ -626,7 +700,7 @@ reeval:
 			/* fall through */
 		case COD_PROFILE_CHAR:
 		case COD_LOCAL_CHAR:
-			if (tolower(this) == COD_UNSAFE_PROFILE_CHAR)
+			if (tolower(thisc) == COD_UNSAFE_PROFILE_CHAR)
 				tmode |= AA_EXEC_PROFILE | AA_MAY_EXEC;
 			else
 			{
@@ -636,7 +710,7 @@ reeval:
 			if (tolower(next) == COD_INHERIT_CHAR) {
 				tmode |= AA_EXEC_INHERIT;
 				if (IS_DIFF_QUAL(mode, tmode)) {
-					yyerror(_("Exec qualifier '%c%c' invalid, conflicting qualifier already specified"), this, next);
+					yyerror(_("Exec qualifier '%c%c' invalid, conflicting qualifier already specified"), thisc, next);
 				} else {
 					mode |= tmode;
 					p += 2;		/* skip x */
@@ -644,13 +718,13 @@ reeval:
 			} else if (tolower(next) == COD_UNSAFE_UNCONFINED_CHAR) {
 				tmode |= AA_EXEC_PUX;
 				if (IS_DIFF_QUAL(mode, tmode)) {
-					yyerror(_("Exec qualifier '%c%c' invalid, conflicting qualifier already specified"), this, next);
+					yyerror(_("Exec qualifier '%c%c' invalid, conflicting qualifier already specified"), thisc, next);
 				} else {
 					mode |= tmode;
 					p += 2;		/* skip x */
 				}
 			} else if (IS_DIFF_QUAL(mode, tmode)) {
-				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"), this);
+				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"), thisc);
 
 			} else {
 				if (next != tolower(next))
@@ -667,7 +741,7 @@ reeval:
 			break;
 
 		case COD_EXEC_CHAR:
-			/* this is valid for deny rules, and named transitions
+			/* thisc is valid for deny rules, and named transitions
 			 * but invalid for regular x transitions
 			 * sort it out later.
 			 */
@@ -677,7 +751,7 @@ reeval:
  		/* error cases */
 
 		default:
-			lower = tolower(this);
+			lower = tolower(thisc);
 			switch (lower) {
 			case COD_READ_CHAR:
 			case COD_WRITE_CHAR:
@@ -686,14 +760,14 @@ reeval:
 			case COD_INHERIT_CHAR:
 			case COD_MMAP_CHAR:
 			case COD_EXEC_CHAR:
-				PDEBUG("Parsing mode: found invalid upper case char %c\n", this);
+				PDEBUG("Parsing mode: found invalid upper case char %c\n", thisc);
 				warn_uppercase();
-				this = lower;
+				thisc = lower;
 				goto reeval;
 				break;
 			default:
 				yyerror(_("Internal: unexpected mode character '%c' in input"),
-					this);
+					thisc);
 				break;
 			}
 			break;
@@ -718,7 +792,82 @@ int parse_mode(const char *str_mode)
 	return mode;
 }
 
-struct cod_entry *new_entry(char *namespace, char *id, int mode, char *link_id)
+static int parse_dbus_sub_mode(const char *str_mode, int *result, int fail, const char *mode_desc __unused)
+{
+	int mode = 0;
+	const char *p;
+
+	PDEBUG("Parsing DBus mode: %s\n", str_mode);
+
+	if (!str_mode)
+		return 0;
+
+	p = str_mode;
+	while (*p) {
+		char current = *p;
+		char lower;
+
+reeval:
+		switch (current) {
+		case COD_READ_CHAR:
+			PDEBUG("Parsing DBus mode: found %s READ\n", mode_desc);
+			mode |= AA_DBUS_RECEIVE;
+			break;
+
+		case COD_WRITE_CHAR:
+			PDEBUG("Parsing DBus mode: found %s WRITE\n",
+			       mode_desc);
+			mode |= AA_DBUS_SEND;
+			break;
+
+		/* error cases */
+
+		default:
+			lower = tolower(current);
+			switch (lower) {
+			case COD_READ_CHAR:
+			case COD_WRITE_CHAR:
+				PDEBUG("Parsing DBus mode: found invalid upper case char %c\n",
+				       current);
+				warn_uppercase();
+				current = lower;
+				goto reeval;
+				break;
+			default:
+				if (fail)
+					yyerror(_("Internal: unexpected DBus mode character '%c' in input"),
+						current);
+				else
+					return 0;
+				break;
+			}
+			break;
+		}
+		p++;
+	}
+
+	PDEBUG("Parsed DBus mode: %s 0x%x\n", str_mode, mode);
+
+	*result = mode;
+	return 1;
+}
+
+int parse_dbus_mode(const char *str_mode, int *mode, int fail)
+{
+	*mode = 0;
+	if (!parse_dbus_sub_mode(str_mode, mode, fail, ""))
+		return 0;
+	if (*mode & ~AA_VALID_DBUS_PERMS) {
+		if (fail)
+			yyerror(_("Internal error generated invalid DBus perm 0x%x\n"),
+				  mode);
+		else
+			return 0;
+	}
+	return 1;
+}
+
+struct cod_entry *new_entry(char *ns, char *id, int mode, char *link_id)
 {
 	struct cod_entry *entry = NULL;
 
@@ -726,7 +875,7 @@ struct cod_entry *new_entry(char *namespace, char *id, int mode, char *link_id)
 	if (!entry)
 		return NULL;
 
-	entry->namespace = namespace;
+	entry->ns = ns;
 	entry->name = id;
 	entry->link_name = link_id;
 	entry->mode = mode;
@@ -750,9 +899,9 @@ struct cod_entry *copy_cod_entry(struct cod_entry *orig)
 	if (!entry)
 		return NULL;
 
-	entry->namespace = orig->namespace ? strdup(orig->namespace) : NULL;
-	entry->name = strdup(orig->name);
-	entry->link_name = orig->link_name ? strdup(orig->link_name) : NULL;
+	DUP_STRING(orig, entry, ns, err);
+	DUP_STRING(orig, entry, name, err);
+	DUP_STRING(orig, entry, link_name, err);
 	entry->mode = orig->mode;
 	entry->audit = orig->audit;
 	entry->deny = orig->deny;
@@ -764,6 +913,10 @@ struct cod_entry *copy_cod_entry(struct cod_entry *orig)
 	entry->next = orig->next;
 
 	return entry;
+
+err:
+	free_cod_entries(entry);
+	return NULL;
 }
 
 void free_cod_entries(struct cod_entry *list)
@@ -772,12 +925,14 @@ void free_cod_entries(struct cod_entry *list)
 		return;
 	if (list->next)
 		free_cod_entries(list->next);
-	if (list->namespace)
-		free(list->namespace);
+	if (list->ns)
+		free(list->ns);
 	if (list->name)
 		free(list->name);
 	if (list->link_name)
 		free(list->link_name);
+	if (list->nt_name)
+		free(list->nt_name);
 	if (list->pat.regex)
 		free(list->pat.regex);
 	free(list);
@@ -797,6 +952,16 @@ void free_mnt_entries(struct mnt_entry *list)
 	free(list);
 }
 
+void free_dbus_entries(struct dbus_entry *list)
+{
+	if (!list)
+		return;
+	if (list->next)
+		free_dbus_entries(list->next);
+
+	free_dbus_entry(list);
+}
+
 static void debug_base_perm_mask(int mask)
 {
 	if (HAS_MAY_READ(mask))
@@ -838,8 +1003,8 @@ void debug_cod_entries(struct cod_entry *list)
 		else
 			printf("\tName:\tNULL\n");
 
-		if (item->namespace)
-			printf("\tNamespace:\t(%s)\n", item->namespace);
+		if (item->ns)
+			printf("\tNs:\t(%s)\n", item->ns);
 
 		if (AA_LINK_BITS & item->mode)
 			printf("\tlink:\t(%s)\n", item->link_name ? item->link_name : "/**");
@@ -847,23 +1012,6 @@ void debug_cod_entries(struct cod_entry *list)
 	}
 }
 
-void debug_flags(struct codomain *cod)
-{
-	printf("Profile Mode:\t");
-
-	if (cod->flags.complain)
-		printf("Complain");
-	else
-		printf("Enforce");
-
-	if (cod->flags.audit)
-		printf(", Audit");
-
-	if (cod->flags.hat)
-		printf(", Hat");
-
-	printf("\n");
-}
 
 static const char *capnames[] = {
 	"chown",
@@ -924,45 +1072,99 @@ void __debug_capabilities(uint64_t capset, const char *name)
 	}
 	printf("\n");
 }
-void debug_capabilities(struct codomain *cod)
+
+/* Bleah C++ doesn't have non-trivial designated initializers so we just
+ * have to make sure these are in order.  This means we are more brittle
+ * but there isn't much we can do.
+ */
+const char *sock_types[] = {
+	"none",		/* 0 */
+	"stream",	/* 1 [SOCK_STREAM] */
+	"dgram",	/* 2 [SOCK_DGRAM] */
+	"raw",		/* 3 [SOCK_RAW] */
+	"rdm",		/* 4 [SOCK_RDM] */
+	"seqpacket",	/* 5 [SOCK_SEQPACKET] */
+	"dccp",		/* 6 [SOCK_DCCP] */
+	"invalid",	/* 7 */
+	"invalid",	/* 8 */
+	"invalid",	/* 9 */
+	"packet",	/* 10 [SOCK_PACKET] */
+	/*
+	 * See comment above
+	*/
+};
+#define ALL_TYPES 0x43e
+
+/* another case of C++ not supporting non-trivial designated initializers */
+#undef AA_GEN_NET_ENT
+#define AA_GEN_NET_ENT(name, AF) name, /* [AF] = name, */
+
+static const char *network_families[] = {
+#include "af_names.h"
+};
+
+void __debug_network(unsigned int *array, const char *name)
 {
-	if (cod->capabilities != 0ull)
-		__debug_capabilities(cod->capabilities, "Capabilities");
-	if (cod->audit_caps != 0ull)
-		__debug_capabilities(cod->audit_caps, "Audit Caps");
-	if (cod->deny_caps != 0ull)
-		__debug_capabilities(cod->deny_caps, "Deny Caps");
-	if (cod->quiet_caps != 0ull)
-		__debug_capabilities(cod->quiet_caps, "Quiet Caps");
-}
+	unsigned int count = sizeof(sock_types)/sizeof(sock_types[0]);
+	unsigned int mask = ~((1 << count) -1);
+	unsigned int i, j;
+	int none = 1;
+	size_t af_max = get_af_max();
 
-void debug_cod_list(struct codomain *cod)
-{
-	if (cod->namespace)
-		printf("Namespace:\t\t%s\n", cod->namespace);
+	for (i = AF_UNSPEC; i < af_max; i++)
+		if (array[i]) {
+			none = 0;
+			break;
+		}
 
-	if (cod->name)
-		printf("Name:\t\t%s\n", cod->name);
-	else
-		printf("Name:\t\tNULL\n");
+	if (none)
+		return;
 
-	if (cod->local)
-		printf("Local To:\t%s\n", cod->parent->name);
+	printf("%s: ", name);
 
-	debug_flags(cod);
-	
-	debug_capabilities(cod);
+	/* This can only be set by an unqualified network rule */
+	if (array[AF_UNSPEC]) {
+		printf("<all>\n");
+		return;
+	}
 
-	if (cod->entries)
-		debug_cod_entries(cod->entries);
+	for (i = 0; i < af_max; i++) {
+		if (array[i]) {
+			const char *fam = network_families[i];
+			if (fam)
+				printf("%s ", fam);
+			else
+				printf("#%u ", i);
 
+			/* All types/protocols */
+			if (array[i] == 0xffffffff || array[i] == ALL_TYPES)
+				continue;
+
+			printf("{ ");
+
+			for (j = 0; j < count; j++) {
+				const char *type;
+				if (array[i] & (1 << j)) {
+					type = sock_types[j];
+					if (type)
+						printf("%s ", type);
+					else
+						printf("#%u ", j);
+				}
+			}
+			if (array[i] & mask)
+				printf("#%x ", array[i] & mask);
+
+			printf("} ");
+		}
+	}
 	printf("\n");
-	dump_policy_hats(cod);
 }
 
+
 struct value_list *new_value_list(char *value)
 {
-	struct value_list *val = calloc(1, sizeof(struct value_list));
+	struct value_list *val = (struct value_list *) calloc(1, sizeof(struct value_list));
 	if (val)
 		val->value = value;
 	return val;
@@ -1028,7 +1230,7 @@ void print_value_list(struct value_list *list)
 
 struct cond_entry *new_cond_entry(char *name, int eq, struct value_list *list)
 {
-	struct cond_entry *ent = calloc(1, sizeof(struct cond_entry));
+	struct cond_entry *ent = (struct cond_entry *) calloc(1, sizeof(struct cond_entry));
 	if (ent) {
 		ent->name = name;
 		ent->vals = list;
@@ -1047,6 +1249,15 @@ void free_cond_entry(struct cond_entry *ent)
 	}
 }
 
+void free_cond_list(struct cond_entry *ents)
+{
+	struct cond_entry *entry, *tmp;
+
+	list_for_each_safe(ents, entry, tmp) {
+		free_cond_entry(entry);
+	}
+}
+
 void print_cond_entry(struct cond_entry *ent)
 {
 	if (ent) {
@@ -1057,6 +1268,9 @@ void print_cond_entry(struct cond_entry *ent)
 }
 
 #ifdef UNIT_TEST
+
+#include "unit_test.h"
+
 int test_str_to_boolean(void)
 {
 	int rc = 0;
@@ -1080,7 +1294,7 @@ int test_str_to_boolean(void)
 int test_processunquoted(void)
 {
 	int rc = 0;
-	char *teststring, *processedstring;
+	const char *teststring, *processedstring;
 
 	teststring = "";
 	MY_TEST(strcmp(teststring, processunquoted(teststring, strlen(teststring))) == 0,
@@ -1108,7 +1322,7 @@ int test_processunquoted(void)
 int test_processquoted(void)
 {
 	int rc = 0;
-	char *teststring, *processedstring;
+	const char *teststring, *processedstring;
 	char *out;
 
 	teststring = "";
