@@ -8,7 +8,7 @@ use Getopt::Long;
 use Test::More;
 
 my %config;
-$config{'parser'} = "/sbin/subdomain_parser";
+$config{'parser'} = "/sbin/apparmor_parser";
 $config{'profiledir'} = "./simple_tests/";
 $config{'timeout'} = 120; # in seconds
 
@@ -30,6 +30,12 @@ sub usage {
 &usage if ($help);
 read_config();
 
+# let environment variable override config file, for use in automated
+# test suites
+if ($ENV{APPARMOR_PARSER}) {
+  $config{'parser'} = $ENV{APPARMOR_PARSER};
+}
+
 # Override config file profile location when passed on command line
 if (@ARGV >= 1) {
   $config{'profiledir'} = shift;
@@ -65,31 +71,32 @@ sub test_profile {
   my $expass = 1;
   my $istodo = 0;
   my $isdisabled = 0;
-  my $result = 0;
+  my $result = 1;
+  my $signal = 0;
+  my $coredump = 0;
   my $child;
 
+  $child = open(PARSER, "|-");
+  if ($child == 0) {
+    # child
+    open(STDOUT, ">/dev/null") or die "Failed to redirect STDOUT";
+    open(STDERR, ">/dev/null") or die "Failed to redirect STDERR";
+    exec("$config{'parser'}", "-S", "-I", "$config{'includedir'}") or die "Bail out! couldn't open parser";
+    # noreturn
+  }
+
+  # parent
   eval {
     local $SIG{ALRM} = sub {
-      $result = 1;
       kill PIPE => $child;
       $description = "$description - TESTCASE TIMED OUT";
     };
 
     alarm $config{'timeout'};
 
-    $child = open(PARSER, "|-");
-    if ($child == 0) {
-      # child
-      open(STDOUT, ">/dev/null") or die "Failed to redirect STDOUT";
-      open(STDERR, ">/dev/null") or die "Failed to redirect STDERR";
-      exec("$config{'parser'}", "-S", "-I", "$config{'includedir'}") or die "Bail out! couldn't open parser";
-      # noreturn
-    }
-
-    # parent
     open(PROFILE, $profile) or die "Bail out! couldn't open profile $profile";
     while (<PROFILE>) {
-      if (/^#=DESCRIPTION\s*(.*)/) {
+      if (/^#=DESCRIPTION\s*(.*)/i) {
         $description = $1;
       } elsif (/^#=EXRESULT\s*(\w+)/) {
         if ($1 eq "PASS") {
@@ -108,7 +115,10 @@ sub test_profile {
       }
     }
 
-    $result = close(PARSER);
+    close(PARSER) or ($! and die "Bail out! couldn't close parser pipe");
+    $result = $? >> 8;
+    $signal = $? & 127;
+    $coredump = $signal ? $? & 128 : 0;
     alarm 0;
   };
 
@@ -118,13 +128,15 @@ sub test_profile {
       local $TODO = "Disabled testcase.";
       ok(0, "TODO: $profile: $description");
     }
+  } elsif ($coredump) {
+    ok(0, "$profile: Produced core dump (signal $signal): $description");
   } elsif ($istodo) {
     TODO: {
       local $TODO = "Unfixed testcase.";
-      ok($expass ? $result : !$result, "TODO: $profile: $description");
+      ok($expass ? !$result : $result, "TODO: $profile: $description");
     }
   } else {
-    ok($expass ? $result : !$result, "$profile: $description");
+    ok($expass ? !$result : $result, "$profile: $description");
   }
 }
 
